package it.arnaldo.unibs.tamagolem;

import it.kibo.fp.lib.InputData;
import it.kibo.fp.lib.Menu;
import it.kibo.fp.lib.PrettyStrings;

import java.util.HashMap;
import java.util.List;
import java.util.Scanner;

public class Game {
    private final Player player1;
    private final Player player2;
    private final ElementGraph worldBalance;
    private int numberOfElements;
    private int numberOfStones;
    private int numberOfTamaGolems;
    private int totalNumberOfStones;
    private int numberOfStonesPerElement;
    private int totalPower;
    private int lifePoints;
    private HashMap<Element, Integer> stonesPerElement;

    public Game(Player player1, Player player2, ElementGraph worldBalance) {
        this.player1 = player1;
        this.player2 = player2;
        this.worldBalance = worldBalance;
        this.stonesPerElement = new HashMap<>();
    }

    public Player getPlayer1() {
        return player1;
    }

    public Player getPlayer2() {
        return player2;
    }

    public ElementGraph getWorldBalance() {
        return worldBalance;
    }

    private void loop(){
        // setup dei tamagolem
        boolean golem1Setupped = setupGolem(player1.getNextTamaGolem(), player1);

        // se viene scelto di uscire
        if (!golem1Setupped) {
            return;
        }

        boolean golem2Setupped = setupGolem(player2.getNextTamaGolem(), player2);

        // se viene scelto di uscire
        if (!golem2Setupped) {
            return;
        }

        // fno a quando uno dei due giocatori non vince
        while(!checkVictory()) {

            FightEsit esito = fight();

            // dopo il fight i tamagolem morti vengono rimossi
            checkDeathsTamagolems(player1, player2);

            switch (esito) {
                case GOLEM1:
                    System.out.println(PrettyStrings.frame(player1.getName() + " won the fight", 30, true, true));
                    next();
                    if(player2.isAlive()){
                        boolean golemSetupped = setupGolem(player2.getNextTamaGolem(), player2);
                        if (!golemSetupped) {
                            return;
                        }
                    }

                    break;
                case GOLEM2:
                    System.out.println(PrettyStrings.frame(player2.getName() + " won the fight", 30, true, true));
                    next();
                    if(player1.isAlive()) {
                        boolean golemSetupped = setupGolem(player1.getNextTamaGolem(), player1);
                        if (!golemSetupped) {
                            return;
                        }
                    }
                    break;
            }

        }

        if (!player1.isAlive()) {
            System.out.println(PrettyStrings.frame(player2.getName() + " won the game", 30, true, true));
        } else {
            System.out.println(PrettyStrings.frame(player1.getName() + " won the game", 30, true, true));
        }

        next();
    }

    private boolean setupGolem(TamaGolem golem, Player player) {
        System.out.println(PrettyStrings.frame("Setting up " + player.getName() + " TamaGolem", 30, true, true));
        System.out.println("You can assign " + numberOfStones + " stones to your tamaGolem");
        next();
        List<Element> elementsNames = worldBalance.getElementsNames();

        for (int i = 0; i < numberOfStones; i++) {
            boolean stoneAssigned = false;
            do {
                Menu stonesMenu = new Menu("Stones available", getMenuOptions(), true, true, false);
                int stonesMenuChoice = stonesMenu.choose();

                if (stonesMenuChoice == 0) {
                    return false;
                }

                Element chosenElement = elementsNames.get(stonesMenuChoice - 1);
                int remaining = stonesPerElement.get(chosenElement);

                if (remaining > 0) {
                    golem.addElementalStone(chosenElement);
                    stonesPerElement.put(chosenElement, remaining - 1);
                    stoneAssigned = true;
                } else {
                    System.out.println(PrettyStrings.frame(chosenElement + " stones are finished, choose another elemental stone", 100, true, true));
                }
            } while (!stoneAssigned);
        }

        System.out.println(player.getName() + " setupped golem: \n\tLP: " + golem.getLifePoints() + "\n\t Stones: " + golem.getElementalStones() + "\n");
        next();
        return true;
    }

    private void checkDeathsTamagolems(Player p1, Player p2) {
        p1.getTamaGolems().removeIf(tg -> !tg.isAlive());
        p2.getTamaGolems().removeIf(tg -> !tg.isAlive());
    }

    private void setup() {
        // N
        numberOfElements = worldBalance.getElements().size();

        // P
        numberOfStones = (int) Math.ceil((numberOfElements + 1) / 3.0) + 1;

        // G
        numberOfTamaGolems = (int) Math.ceil(((numberOfElements - 1) * (numberOfElements - 2)) / (2.0 * numberOfStones));

        // S
        totalNumberOfStones = (int) Math.ceil((2.0 * numberOfTamaGolems * numberOfStones));

        // Stones for each element
        numberOfStonesPerElement = (int) Math.ceil((double)totalNumberOfStones / (double)numberOfElements);


        // deve essere uguale alla somma della potenza di tutti gli elementi
        totalPower = worldBalance.getTotalStrength();

        lifePoints = totalPower;

        List<Element> elementsNames = worldBalance.getElementsNames();

        // setto la mappa con le pietre ancora disponibili
        for (Element element : elementsNames) {
            stonesPerElement.put(element, numberOfStonesPerElement);
        }

        // aggiungo i tamagolem
        System.out.println(numberOfTamaGolems);
        for (int i = 0; i < numberOfTamaGolems; i++) {
            player1.addTamaGolem(new TamaGolem(lifePoints));
            player2.addTamaGolem(new TamaGolem(lifePoints));
        }

    }

    public void start() {
        setup();
        loop();
        worldBalance.printGraph();
        next();
    }

    private boolean checkVictory(){
        return !(player1.isAlive() && player2.isAlive());
    }

    private String[] getMenuOptions() {
        List<Element> elementsNames = worldBalance.getElementsNames();
        String[] menuOptions = new String[numberOfElements];

        for (int i = 0; i < numberOfElements; i++) {
            menuOptions[i] = elementsNames.get(i) + " -> (" + stonesPerElement.get(elementsNames.get(i)) + ")";
        }

        return menuOptions;
    }

    public FightEsit fight() {
        TamaGolem golem1 = player1.getNextTamaGolem();
        TamaGolem golem2 = player2.getNextTamaGolem();

        while (golem1.isAlive() && golem2.isAlive()) {
            Element firstStone = golem1.useElementalStone();
            Element secondStone = golem2.useElementalStone();
            int interaction = worldBalance.getInteractionBetween(firstStone, secondStone);

            System.out.println(player1.getName() + "'s tamagolem throws " + firstStone.name() + " and " + player2.getName() + "'s tamagolem throws " + secondStone.name());
            next();
            if (interaction > 0){
                golem2.getDamage(interaction);
                System.out.println(player2.getName() + "'s tamagolem gets " + interaction + " damage");
            } else {
                golem1.getDamage(-1 * interaction);
                System.out.println(player1.getName() + "'s tamagolem gets " + -1 * interaction + " damage");
            }
            next();
        }

        if (golem1.isAlive()){
            return FightEsit.GOLEM1;
        }
        return FightEsit.GOLEM2;
    }

    public void next(){
        Scanner reader = new Scanner(System.in);
        System.out.print("Press ENTER to continue... ");
        reader.nextLine();
    }
;}
